# -*- coding: utf-8 -*-

import sys
import StringIO
import unittest

from grass.script import (run_command, parse_command,
                          read_command, write_command, write_read_command,
                          CalledCommandError)
import grass.script.core as gcore


class HigherCommandFunctionsTest(unittest.TestCase):
    # this test cannot run in parallel with other imported tests

    def setUp(self):
        self.raise_error = gcore._RAISE_ON_COMMAND_ERROR
        self.return_error = gcore._RETURN_ON_COMMAND_ERROR

    def test_run_command_return(self):
        gcore._RETURN_ON_COMMAND_ERROR = True
        self.assertNotEquals(run_command('r.info', aaabbbcc='notexist'), 0)
        self.assertEquals(run_command('r.info', map='elevation'), 0)

    def test_run_command_exit(self):
        gcore._RAISE_ON_COMMAND_ERROR = False
        self.assertRaises(SystemExit,
                          run_command,
                          'r.univar', aaabbbcc='notexist')

    def test_run_command_raise(self):
        gcore._RAISE_ON_COMMAND_ERROR = True
        self.assertRaises(CalledCommandError,
                          run_command,
                          'r.univar', aaabbbcc='notexist')

    def test_read_command_return(self):
        gcore._RAISE_ON_COMMAND_ERROR = True
        self.assertTrue(read_command('r.info', map='elevation'))

    def test_read_command_exit(self):
        gcore._RAISE_ON_COMMAND_ERROR = False
        self.assertRaises(SystemExit,
                          read_command,
                          'r.univar', aaabbbcc='notexist')

    def test_read_command_raise(self):
        gcore._RAISE_ON_COMMAND_ERROR = True
        self.assertRaises(CalledCommandError,
                          read_command,
                          'r.univar', aaabbbcc='notexist')

    def test_write_command_stdin(self):
        gcore._RAISE_ON_COMMAND_ERROR = True
        self.assertRaises(ValueError,
                          write_command,
                          'm.proj', aaabbbcc='notexist')

    def test_write_command_return(self):
        gcore._RETURN_ON_COMMAND_ERROR = True
        self.assertEquals(write_command('m.proj', input='-', stdin='50,50',
                                        flags='o'), 0)
        self.assertNotEquals(write_command('m.proj', input='-', stdin='50,50',
                                           aaabbbcc='notexist'), 0)

    def test_write_command_exit(self):
        gcore._RAISE_ON_COMMAND_ERROR = False
        self.assertRaises(SystemExit,
                          write_command, 'm.proj',
                          input='-', stdin='50,50', aaabbbcc='notexist')

    def test_write_command_raise(self):
        gcore._RAISE_ON_COMMAND_ERROR = True
        self.assertRaises(CalledCommandError,
                          write_command,
                          'm.proj', stdin='50,50', aaabbbcc='notexist')

    def test_write_read_command_stdin(self):
        gcore._RAISE_ON_COMMAND_ERROR = True
        self.assertRaises(ValueError,
                          write_read_command,
                          'm.proj', aaabbbcc='notexist')

    def test_write_read_command_return(self):
        gcore._RETURN_ON_COMMAND_ERROR = True
        self.assertTrue(write_read_command('m.proj', input='-', stdin='50,50',
                                           flags='o'))

    def test_write_read_command_exit(self):
        gcore._RAISE_ON_COMMAND_ERROR = False
        self.assertRaises(SystemExit,
                          write_read_command, 'm.proj',
                          input='-', stdin='50,50', aaabbbcc='notexist')

    def test_write_read_command_raise(self):
        gcore._RAISE_ON_COMMAND_ERROR = True
        self.assertRaises(CalledCommandError,
                          write_read_command,
                          'm.proj', stdin='50,50', aaabbbcc='notexist')

    def test_parse_command(self):
        self.assertTrue(parse_command('r.info', map='elevation', flags='g'))
        gcore._RAISE_ON_COMMAND_ERROR = False
        self.assertRaises(SystemExit,
                          parse_command,
                          'r.info', map='elevation', aaabbbcc='notexist')
        gcore._RAISE_ON_COMMAND_ERROR = True
        self.assertRaises(CalledCommandError,
                          parse_command,
                          'r.info', aaabbbcc='notexist', map='elevation')

    def tearDown(self):
        gcore._RAISE_ON_COMMAND_ERROR = self.raise_error
        gcore._RETURN_ON_COMMAND_ERROR = self.return_error


if __name__ == '__main__':
    unittest.main()
